import logging
import pathlib
from subprocess import call
logger = logging.getLogger('django')

def resizeImage(imgPath:str):
    cmds:list = []      
    existingPath = pathlib.Path(imgPath)
    destPath = existingPath.parent
    fileName = existingPath.stem
    quality = 100
    mdWidth =  512
    smWidth = 320
    cmdLg='cwebp \"'+imgPath+'\" -q '+str(quality)+' -o '+ str(pathlib.PurePath(destPath,fileName+'_lg.webp'))
    cmdMd='cwebp \"'+imgPath+'\" -q '+str(quality)+' -resize '+str(mdWidth)+' 0 -o '+ str(pathlib.PurePath(destPath,fileName+'_md.webp'))
    cmdSm='cwebp \"'+imgPath+'\" -q '+str(quality)+' -resize '+str(smWidth)+' 0 -o '+ str(pathlib.PurePath(destPath,fileName+'_sm.webp'))

    cmds.append(cmdLg)
    cmds.append(cmdMd)
    cmds.append(cmdSm)

    for comm in cmds:
        call(comm, shell=True)


from django.db.models.signals import post_save
from django.dispatch import receiver
from humans.helpers.images import resizeImage
from humans.models import Human

import logging
logger = logging.getLogger('django')

@receiver(post_save, sender=Human)
def HumanPostSaveHandler(sender, instance:Human, **kwargs):
    try:
        resizeImage (instance.image.path)
    except Exception as ex:  # to handle initial object creation
        logger.error(f"Signal error on portfolio post {str(instance.pk)}:{ex.__str__()}")
        return None  # just exiting from signal


from django.apps import AppConfig


class HumansConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'humans'
    def ready(self) -> None:
        import humans.signals.humans



# Generated by Django 4.2.1 on 2023-05-15 08:19

import ckeditor_uploader.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Human',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('position', models.SmallIntegerField(default=1, help_text='In which position is person listed', unique=True)),
                ('image', models.ImageField(default='humansImages/default.webp', upload_to='humansImages/', verbose_name='Person image')),
                ('name', models.TextField(null=True)),
                ('role', models.TextField(help_text='Role inside of the organization', null=True)),
                ('shortDescription', ckeditor_uploader.fields.RichTextUploadingField(blank=True, null=True)),
                ('description', ckeditor_uploader.fields.RichTextUploadingField(blank=True, null=True)),
                ('isPublished', models.BooleanField(default=False, help_text='After setting Yes, person wil be visible for all users, after setting No - for admins only ', null=True, verbose_name='Published')),
                ('customSlug', models.TextField(help_text='Slug that will lead to persons page, i.e. yourdomain.com/team/<slug>/', unique=True, verbose_name='Custom slug')),
                ('websiteUrl', models.URLField(blank=True, null=True)),
                ('twitterUrl', models.URLField(blank=True, null=True)),
                ('linkedInUrl', models.URLField(blank=True, null=True)),
            ],
            options={
                'verbose_name_plural': 'Humans',
            },
        ),
    ]



# Generated by Django 4.2.5 on 2023-09-12 10:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('humans', '0002_human_metadescription'),
    ]

    operations = [
        migrations.AlterField(
            model_name='human',
            name='metaDescription',
            field=models.TextField(blank=True, null=True),
        ),
    ]



from django.conf import settings

def getBaseTemplate(context:dict):
    context["baseTemplate"] = "base.html"
    if hasattr(settings, "HUMANS_BASE_TEMPLATE"):
        context["baseTemplate"] = settings.HUMANS_BASE_TEMPLATE


from django.urls import path, include
from humans.views import teamPageView, personView, humans_txt

urlpatterns = [
    path('team/', teamPageView, name="humansTeamView"),
    path('team/<slug:personSlug>/', personView, name="humansPersonPageView"),
    path("humans.txt", humans_txt),
]


from django.contrib import sitemaps
from humans.models import Human

class HumansSitemap(sitemaps.Sitemap):
    protocol = "https"
    priority = 0.5
    changefreq = 'daily'

    def items(self):
        return Human.objects.filter(isPublished=True)



from django.http import HttpRequest
from django.db.models import QuerySet
from humans.models import Human

def getTeam(request:HttpRequest) -> QuerySet:
    team:QuerySet
    if request.user.is_superuser:
        team:QuerySet = Human.objects.all().order_by("position")
    else:
        team:QuerySet = Human.objects.filter(isPublished=True).order_by("position")
    return team


from pathlib import Path, PurePath
from django import template
from django.db.models.fields.files import ImageFieldFile

register = template.Library()

@register.filter(is_safe=True)
def getOptimizedImage(img, rez:str, ext:str):
    imagePath = Path(img.url)
    fileName = imagePath.stem
    parent = imagePath.parent
    return str(PurePath(parent,fileName+rez+ext))

@register.filter(is_safe=True)
def srcSet(image:ImageFieldFile):
    if (image):
        returnString = f"""
            src="{getOptimizedImage(image, "_sm", ".webp")}" 
            srcset="{getOptimizedImage(image, "_lg", ".webp")} 1024w, 
                    {getOptimizedImage(image, "_md", ".webp")} 640w, 
                    {getOptimizedImage(image, "_sm", ".webp") } 360w" 
            loading="lazy" 
            sizes="100vw" """
    else:
        returnString = ""
    return returnString


from django.db import models
from ckeditor_uploader.fields import RichTextUploadingField
from django.utils.translation import gettext as _
from django.urls import reverse
# Create your models here.

class Human(models.Model):
    position = models.SmallIntegerField(unique=True, help_text="In which position is person listed", default=1)
    image = models.ImageField(upload_to='humansImages/', default='humansImages/default.webp', verbose_name=_("Person image"))
    name = models.TextField(null=True)
    role = models.TextField(null=True, help_text=_("Role inside of the organization"))
    shortDescription = RichTextUploadingField(null=True, blank=True)
    description = RichTextUploadingField(null=True, blank=True)
    isPublished = models.BooleanField(default=False, null=True, verbose_name="Published", help_text=_("After setting Yes, person wil be visible for all users, after setting No - for admins only "))
    customSlug = models.TextField(blank=False, null=False, verbose_name=_("Custom slug"), help_text=_("Slug that will lead to persons page, i.e. yourdomain.com/team/<slug>/"), unique=True)
    websiteUrl = models.URLField(null=True, blank=True)
    twitterUrl = models.URLField(null=True, blank=True)
    linkedInUrl = models.URLField(null=True, blank=True)
    metaTitle = models.TextField(null=True, blank=True)
    metaDescription = models.TextField(null=True, blank=True)
    class Meta:
        verbose_name_plural = "Humans"
    
    def get_absolute_url(self):
        return reverse('humansPersonPageView', kwargs={'personSlug': self.customSlug})


from django.http import Http404, HttpRequest, HttpResponse, HttpResponseNotFound
from django.shortcuts import render
from django.conf import settings
from humans.models import Human
from humans.helpers.team import getTeam
from humans.helpers.generic import getBaseTemplate
from django.db.models import QuerySet
from django.views.decorators.http import require_GET
from django.utils.html import strip_tags

def teamPageView(request:HttpRequest):
    context:dict = {}
    getBaseTemplate(context)
    team:QuerySet = getTeam(request)
    if (team.__len__() > 0 or request.user.is_superuser):
        context["team"] = team
        return render(request, 'humans/teamPage.html', context=context)
    else:
        return HttpResponseNotFound()

def personView (request:HttpRequest, personSlug:str):
    context:dict = {}
    getBaseTemplate(context)
    try:
        person:Human = Human.objects.get(customSlug=personSlug)
        context["person"] = person
        return render(request, 'humans/personPage.html', context=context)
    except Human.DoesNotExist:
        return HttpResponseNotFound()

@require_GET
def humans_txt(request):
    response:str = "/* TEAM */\n\n"
    team:QuerySet = getTeam(request)
    if (team.__len__() > 0 or request.user.is_superuser):
        person:Human
        for person in team:
            personString:str = "--------------------------------------------\n"
            personString = personString + f"""Title: {person.position}\nName: {person.name}\nSite: {person.websiteUrl}\nTwitter: {person.twitterUrl}\nLinkedIn: {person.linkedInUrl}\nAbout:\n{strip_tags(person.shortDescription)}\n"""
            personString = personString + "--------------------------------------------\n"
            response = response + personString
        return HttpResponse (response, content_type="text/plain")
    else:
        return HttpResponseNotFound()


# Generated by Django 4.2.5 on 2023-09-12 08:20

import ckeditor_uploader.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('humans', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='human',
            name='metaDescription',
            field=ckeditor_uploader.fields.RichTextUploadingField(blank=True, null=True),
        ),
    ]
